import 'package:flutter/foundation.dart';
import 'package:puzzlegame/entities/moving_tile.dart';

class Data extends ChangeNotifier {
  // application configuration
  int _tilesCount = 4;

  // application data
  List _availableImages = [];
  String _selectedImage = '';
  List<MovingTile> _tiles = [];

  // application state
  bool _isShufflingBoard = false;
  bool _isTipImageDisplayed = false;

  String get selectedImage => _selectedImage;

  void updateSelectedImage(String value) {
    _selectedImage = value;
    notifyListeners();
  }

  void resetSelectedImage() {
    updateSelectedImage('');
  }

  int get tilesCount => _tilesCount;

  void updateTilesCount(int tilesCountPerSide) {
    _tilesCount = tilesCountPerSide;
    notifyListeners();
  }

  List get availableImages => _availableImages;

  void updateAvailableImages(List availableImages) {
    _availableImages = availableImages;
    notifyListeners();
  }

  List<MovingTile> get tiles => _tiles;

  void updateTiles(List<MovingTile> tiles) {
    _tiles = tiles;
    notifyListeners();
  }

  bool get isShufflingBoard => _isShufflingBoard;

  void updateIsShufflingBoard(bool isShuffling) {
    _isShufflingBoard = isShuffling;
    notifyListeners();
  }

  bool get displayTipImage => _isTipImageDisplayed;

  void updateIsTipImageDisplayed(bool isDisplayed) {
    _isTipImageDisplayed = isDisplayed;
    notifyListeners();
  }

  void swapTiles(List<int> tile1, List<int> tile2) {
    int indexTile1 = _tiles.indexWhere(
        (tile) => ((tile.currentCol == tile1[0]) && (tile.currentRow == tile1[1])));
    int indexTile2 = _tiles.indexWhere(
        (tile) => ((tile.currentCol == tile2[0]) && (tile.currentRow == tile2[1])));

    MovingTile swap = _tiles[indexTile1];
    _tiles[indexTile1] = _tiles[indexTile2];
    _tiles[indexTile2] = swap;

    int swapCol = _tiles[indexTile1].currentCol;
    _tiles[indexTile1].currentCol = _tiles[indexTile2].currentCol;
    _tiles[indexTile2].currentCol = swapCol;

    int swapRow = _tiles[indexTile1].currentRow;
    _tiles[indexTile1].currentRow = _tiles[indexTile2].currentRow;
    _tiles[indexTile2].currentRow = swapRow;

    notifyListeners();
  }
}
